// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2012 Marco Drache, Georg Drache
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

/*
 * mcPolymer.cpp
 *
 *  Created on: 24.08.2008
 * 	Author: drache
 */

// Linux Version 
#include "/usr/include/tcl.h"
// Windows Version
//#include "C:\Program Files\Tcl\include\tcl.h"
// Option: google-perftools profiler
//#include <google/profiler.h>
#include "MersenneTwister.h"
#include "mcPolymer.h"

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <math.h>
#include <time.h>
#include <ctime>

Reactant::Reactant(int _id, double _molcount, bool _macro, int _type,
		double _addvalue) {
	id = _id;
	molcount = _molcount;
	macro = _macro;
	type = _type;

	macroH = false;
	macroD = false;

	if (type == 102) {
		macroD = true;
	}
	if (type == 103) {
		macroH = true;
	}

	addvalue = _addvalue;
	if (macro) {
		if (macroD) {
			poly = new polymerSpecies(macroD);
		} else {
			if (macroH) {
				poly = new polymerSpecies(0.0);
			} else {
				poly = new polymerSpecies();
			}
		}
	}
}
MCRateC::MCRateC(int _id, double _value) {
	id = _id;
	value = _value;
}
Reaction::Reaction(int _id, MCRateC* _k, int _type) {
	id = _id;
	k = _k;
	type = _type;

}

bool CoreInterface::addReactant(int _id, double _molcount, bool _macro,
		int _type, double _addvalue) {
	vector<Reactant*>::iterator I;
	Reactant *rp;
	polymerSpecies* polys;
	for (I = Reactants.begin(); I != Reactants.end(); ++I) {
		rp = *I;
		if (rp->getID() == _id) {
			return false;
		}
	}
	Reactants.insert(Reactants.end(),
			new Reactant(_id, _molcount, _macro, _type, _addvalue));
	if (_macro) {
		if (_molcount > 0.0) {
			rp = Reactants[Reactants.size() - 1];
			if (rp->isMacroD() == false) {
				polys = rp->getPolymerS();
				polys->readDistributionBinary(0, 0, _id);
			}
		}
	}
	return true;
}
bool CoreInterface::addReaction(int _id, int _idMCRateC, int _type) {
	vector<Reaction*>::iterator I;
	Reaction *rp;
	vector<MCRateC*>::iterator J;
	MCRateC *mcRp;
	for (I = Reactions.begin(); I != Reactions.end(); ++I) {
		rp = *I;
		if (rp->getID() == _id) {
			return false;
		}
	}

	for (J = MCRateCs.begin(); J != MCRateCs.end(); ++J) {
		mcRp = *J;
		if (mcRp->getID() == _idMCRateC) {
			Reactions.insert(Reactions.end(), new Reaction(_id, mcRp, _type));
			return true;
		}
	}
	return false;
}
bool CoreInterface::addMCRateC(int _id, double _value) {
	vector<MCRateC*>::iterator I;
	MCRateC *mcRp;
	for (I = MCRateCs.begin(); I != MCRateCs.end(); ++I) {
		mcRp = *I;
		if (mcRp->getID() == _id) {
			return false;
		}
	}
	MCRateCs.insert(MCRateCs.end(), new MCRateC(_id, _value));
	return true;
}
bool CoreInterface::setMCRateC(int _id, double _value) {
	vector<MCRateC*>::iterator I;
	MCRateC *mcRp;
	for (I = MCRateCs.begin(); I != MCRateCs.end(); ++I) {
		mcRp = *I;
		if (mcRp->getID() == _id) {
			mcRp->setValue(_value);
			return true;
		}
	}
	return false;
}
void Reaction::addEduct(Reactant* rp) {
	educt.insert(educt.end(), rp);
}

void Reaction::addProduct(Reactant* pp) {
	product.insert(product.end(), pp);
}

Polymer::Polymer(chain *c) {
	// linear Polymer
	polymerChain.push_back(c);
	conserveChains = false;
}
Polymer::Polymer(chain *c1, chain *c2) {
	// linear Polymer P1-X-P2
	polymerChain.push_back(c1);
	polymerChain.push_back(c2);
	conserveChains = false;
}
Polymer::Polymer(int nchains, chain **c1) {
	int i;
	// Polymer with n chains
	for (i = 0; i < nchains; i++) {
		polymerChain.push_back(c1[i]);
	}
	conserveChains = false;
}
Polymer::Polymer(chain *c1, chain *c2, chain *c3) {
	// star Polymer P1-X<P2P3
	polymerChain.push_back(c1);
	polymerChain.push_back(c2);
	polymerChain.push_back(c3);
	conserveChains = false;
}
Polymer::Polymer(chain *c1, chain *c2, chain *c3, chain *c4) {
	// star Polymer P4P1>X<P2P3
	polymerChain.push_back(c1);
	polymerChain.push_back(c2);
	polymerChain.push_back(c3);
	polymerChain.push_back(c4);
	conserveChains = false;
}

void Simulation::updateMCRates(CoreInterface *_ci) {
	int i;
	Reaction *rp;
	ci = _ci;
	for (i = 0; i < ci->Reactions.size(); i++) {
		rp = ci->Reactions[i];
		rkt[i].k = rp->k->getValue();
	}
}

Simulation::Simulation(CoreInterface *_ci) {
	int i, j, k, l;
	Reaction *rp;
	ci = _ci;
	double test;

	rktSize = ci->Reactions.size();
	pr = new double[ci->Reactions.size()];
	cPr = new double[ci->Reactions.size()];
	cReactants = ci->Reactants.size();
	reactants = new double[ci->Reactants.size()];
	for (i = 0; i < ci->Reactants.size(); i++) {
		reactants[i] = ci->Reactants[i]->getMolcount();
	}
	rkt = new FRkt[ci->Reactions.size()];
	for (i = 0; i < ci->Reactions.size(); i++) {
		rp = ci->Reactions[i];
		rkt[i].k = rp->k->getValue();
		rkt[i].cEducts = rp->educt.size();
		rkt[i].cProducts = rp->product.size();
		rkt[i].educts = new double*[rp->educt.size()];
		rkt[i].polyeducts = new polymerSpecies*[rp->educt.size()];

		rkt[i].type = rp->getType();
		rkt[i].self = false;
		if (rp->educt.size() == 2) {
			if (rp->educt[0]->getID() == rp->educt[1]->getID()) {
				rkt[i].self = true;
			}
		}
		for (j = 0; j < rp->educt.size(); j++) {
			for (k = 0; k < ci->Reactants.size(); k++) {
				if (rp->educt[j]->getID() == ci->Reactants[k]->getID()) {
					break;
				}
			}
			if (rkt[i].type == 1) {
				// Initiator Decomposition
				if (j == 0) {
					// Initiator
					rkt[i].addvalue = ci->Reactants[k]->getAddvalue();

				}
			}
			rkt[i].educts[j] = &reactants[k];
			rkt[i].polyeducts[j] = ci->Reactants[k]->getPolymerS();
		}
		rkt[i].products = new double*[rp->product.size()];
		rkt[i].polyproducts = new polymerSpecies*[rp->product.size()];
		for (j = 0; j < rp->product.size(); j++) {
			for (k = 0; k < ci->Reactants.size(); k++) {
				if (rp->product[j]->getID() == ci->Reactants[k]->getID()) {
					break;
				}
			}
			rkt[i].products[j] = &reactants[k];
			rkt[i].polyproducts[j] = ci->Reactants[k]->getPolymerS();
		}
	}
	rktUpdate = new bool[ci->Reactions.size()];
	for (i = 0; i < ci->Reactions.size(); i++) {
		rktUpdate[i] = true;
		rkt[i].rktUpdate = new bool[ci->Reactions.size()];
		for (j = 0; j < ci->Reactions.size(); j++) {
			rkt[i].rktUpdate[j] = false;
			for (k = 0; k < rkt[i].cEducts; k++) {
				for (l = 0; l < rkt[j].cEducts; l++) {
					if (rkt[i].educts[k] == rkt[j].educts[l]) {
						rkt[i].rktUpdate[j] = true;
					}
				}
			}
			for (k = 0; k < rkt[i].cProducts; k++) {
				for (l = 0; l < rkt[j].cEducts; l++) {
					if (rkt[i].products[k] == rkt[j].educts[l]) {
						rkt[i].rktUpdate[j] = true;
					}
				}
			}
		}
	}
	t = 0.0;
}
int Simulation::runTo(int _cmdCount, double _destTime, bool modeMacro,
		bool modeCopo, bool modeHomopolymerization) {
	srand (time(NULL) );MTRand mtrand1(rand() % 100);
	MTRand mtrand2(rand() % 100);
	double randr, sumPr, pri, csum;
	int i, j, type;
	char buffer[100];
	int slength;
	FRkt *rkti;
	Polymer *p1;
	Polymer *p2;
	Copolymer *cop1;
	Copolymer *cop2;
	polymerSpecies* polyS;
	int cl1, cl2;
	double mcsteps;
	
	// Option: google-perftools
	// ProfilerStart("mcpolymer.prof");
	mcsteps = 0.0;
	while (t < _destTime) {
		sumPr = 0.0;
		rkti = &rkt[0];
		for (i = 0; i < rktSize; i++) {
			if (rktUpdate[i])
			{
				rkti = &rkt[i];
				pri = *rkti->educts[0];
				if (rkti->self) {
					pri *= (*rkti->educts[0] - 1.0)/2.0;
				} else {
					for (j = 1; j < rkti->cEducts; j++) {
						pri *= *rkti->educts[j];
					}
				}
				pri = pri * rkti->k;
				pr[i] = pri;
			}
			sumPr = sumPr + pr[i];
		}
		randr = mtrand1() * sumPr;
		csum = 0.0;
		for (i = 0; i < rktSize; i++) {
			csum = csum + pr[i];
			if (randr <= csum)
			break;
		}
		rkti = &rkt[i];
		for (i = 0; i < rkti->cEducts; i++) {
			*rkti->educts[i] -= 1.0;
		}
		if (rkti->type == 1) {
			// Initiator decomposition
			for (i = 0; i < rkti->cProducts; i++) {
				randr = mtrand1();
				if (randr < rkti->addvalue) {
					*rkti->products[i] += 1.0;
				}
			}
		} else {
			for (i = 0; i < rkti->cProducts; i++) {
				*rkti->products[i] += 1.0;
			}
		}
		rktUpdate = rkti->rktUpdate;
		if (modeHomopolymerization)
		{
			// macromolecular reactions
			if (rkti->type > 0) {
				switch (rkti->type) {

					case 2: {
						// propagation
						rkti->polyeducts[0]->incrP_HP();
						break;
					}
					case 3: {
						// termination 2P
						cl1 = rkti->polyeducts[0]->deleteP_HP();
						cl2 = rkti->polyeducts[0]->deleteP_HP();
						rkti->polyproducts[0]->newDeadP_HP(cl1, cl2);
						break;
					}

					case 4: {
						// transfer2monomer
						cl1 = rkti->polyeducts[0]->deleteP_HP();
						rkti->polyproducts[0]->newDeadP_HP(cl1);
						rkti->polyproducts[1]->newP1_HP();
						break;
					}
					case 5: {
						// thermalInitiation
						rkti->polyproducts[0]->newP1_HP();
						rkti->polyproducts[0]->newP1_HP();
						break;
					}

					case 6: {
						// initiation
						rkti->polyproducts[0]->newP1_HP();
						break;
					}

					case 7: {
						// Transfer_XX
						cl1 = rkti->polyeducts[0]->deleteP_HP();
						rkti->polyproducts[0]->newP_HP(cl1);
						break;
					}
					case 8: {
						// TerminationD
						cl1 = rkti->polyeducts[0]->deleteP_HP();
						cl2 = rkti->polyeducts[1]->deleteP_HP();
						rkti->polyproducts[0]->newDeadP_HP(cl1, cl2);
						break;
					}
				}
			}
		}
		else
		{
			if (modeMacro) {
				// macromolecular reactions
				if (rkti->type > 0) {
					switch (rkti->type) {

						case 2: {
							// propagetion
							rkti->polyeducts[0]->incrP();
							break;
						}
						case 3: {
							// termination 2P
							p1 = rkti->polyeducts[0]->deleteP();
							p2 = rkti->polyeducts[0]->deleteP();
							rkti->polyproducts[0]->newDeadP(p1, p2);
							break;
						}

						case 4: {
							// transfer2monomer
							p1 = rkti->polyeducts[0]->deleteP();
							rkti->polyproducts[0]->newDeadP(p1);
							rkti->polyproducts[1]->newP1();
							break;
						}
						case 5: {
							// thermalInitiation
							rkti->polyproducts[0]->newP1();
							rkti->polyproducts[0]->newP1();
							break;
						}

						case 6: {
							// initiation
							rkti->polyproducts[0]->newP1();
							break;
						}

						case 7: {
							// Transfer_XX
							p1 = rkti->polyeducts[0]->deleteP();
							rkti->polyproducts[0]->newPfromPolymer(p1);
							break;
						}
						case 8: {
							// TerminationD
							p1 = rkti->polyeducts[0]->deleteP();
							p2 = rkti->polyeducts[1]->deleteP();
							rkti->polyproducts[0]->newDeadP(p1, p2);
							break;
						}
						case 9: {
							// ChainAddition
							p1 = rkti->polyeducts[0]->deleteP();
							p2 = rkti->polyeducts[1]->deleteP();
							rkti->polyproducts[0]->newPplusX(p1, p2);
							break;
						}
						case 10: {
							// ChainFractionation
							p1 = rkti->polyeducts[0]->deleteP();
							do {
								randr = mtrand1();
								if (randr < 1.0) {break;}
							}
							while (true);
							p2 = p1->fracPolymer(randr);
							rkti->polyproducts[0]->newPfromPolymer(p1);
							rkti->polyproducts[1]->newPfromPolymer(p2);
							break;
						}
					}
				}
			}
		}

		if (modeCopo) {
			// Copolymerization
			if (rkti->type > 0) {
				switch (rkti->type) {
					case 21: {
						// InitiationCopolymer11
						rkti->polyproducts[0]->newCP(2,0);
						break;
					}
					case 22: {
						// InitiationCopolymer12
						rkti->polyproducts[0]->newCP(1,1);
						break;
					}
					case 23: {
						// InitiationCopolymer21
						rkti->polyproducts[0]->newCP(1,1);
						break;
					}
					case 24: {
						// InitiationCopolymer22
						rkti->polyproducts[0]->newCP(0,2);
						break;
					}
					case 25: {
						// PropagationCopolymer1
						cop1 = rkti->polyeducts[0]->deleteCP();
						cop1->addM1();
						rkti->polyproducts[0]->newCP(cop1);
						break;
					}
					case 26: {
						// PropagationCopolymer2
						cop1 = rkti->polyeducts[0]->deleteCP();
						cop1->addM2();
						rkti->polyproducts[0]->newCP(cop1);
						break;
					}
					case 27: {
						// TransferCopolymer_XX
						cop1 = rkti->polyeducts[0]->deleteCP();
						rkti->polyproducts[0]->newCP(cop1);
						break;
					}
					case 28: {
						// TerminationDispCopolymer
						cop1 = rkti->polyeducts[0]->deleteCP();
						cop2 = rkti->polyeducts[1]->deleteCP();
						rkti->polyproducts[0]->newCP(cop1);
						rkti->polyproducts[1]->newCP(cop2);
						break;
					}
					case 29: {
						// TerminationCombCopolymer
						cop1 = rkti->polyeducts[0]->deleteCP();
						cop2 = rkti->polyeducts[1]->deleteCP();
						rkti->polyproducts[0]->newDeadCP(cop1, cop2);
						break;
					}
				}
			}
		}
		mcsteps = mcsteps + 1.0;
		randr = mtrand2()+1.0e-40;
		t -= log(randr) / sumPr;

	}
	//cout << mcsteps << endl;
	// Option google-perftools
	// ProfilerStop();
	slength = sprintf(buffer, "result.%u", _cmdCount);
	ofstream resFile(buffer);
	slength = sprintf(buffer, "%8.6e\n", t);
	resFile.write(buffer, slength);
	for (i = 0; i < cReactants; i++) {
		slength = sprintf(buffer, "%u\t%8.8e\n", ci->Reactants[i]->getID(),
				reactants[i]);
		resFile.write(buffer, slength);
	}
	resFile.close();
	if (modeMacro) {
		for (i = 0; i < ci->countReactants(); i++) {
			if (ci->Reactants[i]->isMacro()) {
				polyS = ci->Reactants[i]->getPolymerS();
				if (ci->Reactants[i]->isMacroD()) {
					polyS->writeDistributionDead(_cmdCount, ci->Reactants[i]->getID());
				}
				else {
					polyS->writeDistribution(_cmdCount, ci->Reactants[i]->getID());
					// Option
					//polyS->writeDistributionBinary(_cmdCount, ci->Reactants[i]->getID());
				}
			}
		}
	}
	if (modeHomopolymerization) {
		for (i = 0; i < ci->countReactants(); i++) {
			if (ci->Reactants[i]->isMacro()) {
				polyS = ci->Reactants[i]->getPolymerS();
				polyS->writeDistributionDead(_cmdCount, ci->Reactants[i]->getID());
			}
		}
	}
	if (modeCopo) {
		for (i = 0; i < ci->countReactants(); i++) {
			if (ci->Reactants[i]->isMacro()) {
				polyS = ci->Reactants[i]->getPolymerS();
				polyS->writeDistributionCP(_cmdCount, ci->Reactants[i]->getID());
			}
		}
	}
	return 1;
}
bool CoreInterface::addEduct2Reaction(int _idReaction, int _idReactant) {
	vector<Reaction*>::iterator I;
	vector<Reactant*>::iterator J;
	Reaction *reactionp;
	Reactant *reactantp;
	for (I = Reactions.begin(); I != Reactions.end(); ++I) {
		reactionp = *I;
		if (reactionp->getID() == _idReaction) {
			for (J = Reactants.begin(); J != Reactants.end(); ++J) {
				reactantp = *J;
				if (reactantp->getID() == _idReactant) {
					reactionp->addEduct(reactantp);
					return true;
				}
			}
		}
	}
	cout << "Reaction/Educt not found!" << endl;
	return false;
}
bool CoreInterface::addProduct2Reaction(int _idReaction, int _idReactant) {
	vector<Reaction*>::iterator I;
	vector<Reactant*>::iterator J;
	Reaction *reactionp;
	Reactant *reactantp;
	for (I = Reactions.begin(); I != Reactions.end(); ++I) {
		reactionp = *I;
		if (reactionp->getID() == _idReaction) {
			for (J = Reactants.begin(); J != Reactants.end(); ++J) {
				reactantp = *J;
				if (reactantp->getID() == _idReactant) {
					reactionp->addProduct(reactantp);
					return true;
				}
			}
		}
	}
	cout << "Reaction/Product not found!" << endl;
	return false;
}

int CoreInterface::countReactants() {
	return Reactants.size();
}
int CoreInterface::countMCRateCs() {
	return MCRateCs.size();
}
int CoreInterface::countReactions() {
	return Reactions.size();
}

int CoreInterface::initSimulation() {
	sim = new Simulation(this);
	double *reactants = new double[Reactants.size()];
	int i;
	Reactant *rp;
	for (i = 0; i < Reactants.size(); i++) {
		reactants[i] = Reactants[i]->getMolcount();
	}
	return 0;
}
void CoreInterface::updateMCRates() {
	sim->updateMCRates(this);
}
int CoreInterface::runSimulation(int _cmdCount, double _destTime) {
	return sim->runTo(_cmdCount, _destTime, modeMacro, modeCopo,
			modeHomopolymerization);
}
CoreInterface *interface = new CoreInterface();

using namespace std;

int Tcl_AppInit(Tcl_Interp *tclip);

static int initSim(ClientData clientData, Tcl_Interp *interp, int objc,
		Tcl_Obj * CONST objv[]) {
	string line, str;
	char buffer[200];
	size_t found, found1, found2, swap, length;
	int cmdc;
	int commandcount, count, i, count1, j, id, id1, idr, xtype;
	double dvalue, addvalue;

	ifstream inFile("init.sim");
	getline(inFile, line);
	found = line.find_first_of("\t", 0);
	length = line.copy(buffer, line.length() - found, found + 1);
	buffer[length] = '\0';
	count = atoi(buffer);
	if (count == 1) {
		interface->modeMacro = true;
	} else {
		interface->modeMacro = false;
	}
	if (count == 2) {
		interface->modeCopo = true;
	} else {
		interface->modeCopo = false;
	}
	if (count == 3) {
		interface->modeHomopolymerization = true;
	} else {
		interface->modeHomopolymerization = false;
	}
	cmdc = 0;
	commandcount = 0;
	while (getline(inFile, line)) {
		if (cmdc == 0) {
			found = line.find_first_of("\t", 0);
			length = line.copy(buffer, line.length() - found, found + 1);
			buffer[length] = '\0';

			count = atoi(buffer);
			for (i = 0; i < count; i++) {
				getline(inFile, line);
				found = line.find_first_of("\t", 0);
				length = line.copy(buffer, found, 0);
				buffer[length] = '\0';
				id = atoi(buffer);
				found1 = line.find_first_of("\t", found + 1);
				length = line.copy(buffer, found1 - found, found + 1);
				buffer[length] = '\0';
				xtype = atoi(buffer);
				length = line.copy(buffer, line.length() - found1, found1 + 1);
				buffer[length] = '\0';
				dvalue = atof(buffer);
				found2 = line.find_first_of("\t", found1 + 1);

				length = line.copy(buffer, line.length() - found2, found2 + 1);
				buffer[length] = '\0';
				addvalue = atof(buffer);
				if (xtype >= 100) {
					interface->addReactant(id, dvalue, true, xtype, addvalue);
				} else {
					interface->addReactant(id, dvalue, false, xtype, addvalue);
				}

			}
			cmdc = 1;
		}
		if (cmdc == 1) {
			getline(inFile, line);
			found = line.find_first_of("\t", 0);
			length = line.copy(buffer, line.length() - found, found + 1);
			buffer[length] = '\0';
			count = atoi(buffer);
			for (i = 0; i < count; i++) {
				getline(inFile, line);
				found = line.find_first_of("\t", 0);
				length = line.copy(buffer, found, 0);
				buffer[length] = '\0';
				id = atoi(buffer);
				length = line.copy(buffer, line.length() - found, found + 1);
				buffer[length] = '\0';
				dvalue = atof(buffer);
				interface->addMCRateC(id, dvalue);
			}
			cmdc = 2;
		}
		if (cmdc == 2) {
			getline(inFile, line);
			found = line.find_first_of("\t", 0);
			length = line.copy(buffer, line.length() - found, found + 1);
			buffer[length] = '\0';
			count = atoi(buffer);
			for (i = 0; i < count; i++) {
				getline(inFile, line);
				found = line.find_first_of("\t", 0);
				length = line.copy(buffer, found, 0);
				buffer[length] = '\0';
				id = atoi(buffer);
				found1 = line.find_first_of("\t", found + 1);
				length = line.copy(buffer, found1 - found, found + 1);
				buffer[length] = '\0';
				xtype = atoi(buffer);
				found = line.find_first_of("\t", found1 + 1);
				length = line.copy(buffer, found - found1, found1 + 1);
				buffer[length] = '\0';
				id1 = atoi(buffer);
				interface->addReaction(id, id1, xtype);
				found1 = line.find_first_of("\t", found + 1);
				length = line.copy(buffer, found1 - found, found + 1);
				buffer[length] = '\0';
				count1 = atoi(buffer);
				for (j = 0; j < count1; j++) {
					found = line.find_first_of("\t", found1 + 1);
					length = line.copy(buffer, found - found1, found1 + 1);
					buffer[length] = '\0';
					idr = atoi(buffer);
					swap = found1;
					found1 = found;
					found = swap;
					interface->addEduct2Reaction(id, idr);
				}
				found = line.find_first_of("\t", found1 + 1);
				length = line.copy(buffer, found - found1, found1 + 1);
				buffer[length] = '\0';
				count1 = atoi(buffer);
				swap = found1;
				found1 = found;
				found = swap;
				for (j = 0; j < count1; j++) {
					found = line.find_first_of("\t", found1 + 1);
					length = line.copy(buffer, found - found1, found1 + 1);
					buffer[length] = '\0';
					idr = atoi(buffer);
					swap = found1;
					found1 = found;
					found = swap;
					interface->addProduct2Reaction(id, idr);
				}
				cmdc = 3;
			}

		}
	}
	inFile.close();
	interface->initSimulation();
	cout << "init Simulation\tok" << endl;
	return TCL_OK;
}
static int gotoTime(ClientData clientData, Tcl_Interp *interp, int objc,
		Tcl_Obj * CONST objv[]) {
	int res;
	int ip;
	double dp;
	res = Tcl_GetIntFromObj(interp, objv[1], &ip);
	res = Tcl_GetDoubleFromObj(interp, objv[2], &dp);
	interface->runSimulation(ip, dp);

	return TCL_OK;
}
static int setk(ClientData clientData, Tcl_Interp *interp, int objc,
		Tcl_Obj * CONST objv[]) {
	int res;
	int ip;
	double dp;
	res = Tcl_GetIntFromObj(interp, objv[1], &ip);
	res = Tcl_GetDoubleFromObj(interp, objv[2], &dp);
	interface->setMCRateC(ip, dp);
	interface->updateMCRates();
	return TCL_OK;
}
static int showw(ClientData clientData, Tcl_Interp *interp, int objc,
		Tcl_Obj * CONST objv[]) {
	cout << "This program is distributed in the hope that it will be useful," << endl;
	cout << "but WITHOUT ANY WARRANTY; without even the implied warranty of" << endl;
	cout << "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the" << endl;
	cout << "GNU General Public License for more details." << endl;
	cout << "You should have received a copy of the GNU General Public License" << endl;
	cout << "along with this program.  If not, see <http://www.gnu.org/licenses/>." << endl;
	return TCL_OK;
}

static int showc(ClientData clientData, Tcl_Interp *interp, int objc,
		Tcl_Obj * CONST objv[]) {
	cout << "This program is free software: you can redistribute it and/or modify" << endl;
	cout << "it under the terms of the GNU General Public License as published by" << endl;
	cout << "the Free Software Foundation, either version 3 of the License, or" << endl;
	cout << "(at your option) any later version." << endl;
	cout << "You should have received a copy of the GNU General Public License" << endl;
	cout << "along with this program.  If not, see <http://www.gnu.org/licenses/>." << endl;
	return TCL_OK;
}

int TCLIF_Init(Tcl_Interp *tclip) {
	Tcl_CreateObjCommand(tclip, "initSim", initSim, NULL, NULL);
	Tcl_CreateObjCommand(tclip, "gotoTime", gotoTime, NULL, NULL);
	Tcl_CreateObjCommand(tclip, "setk", setk, NULL, NULL);
	Tcl_CreateObjCommand(tclip, "show-w", showw, NULL, NULL);
	Tcl_CreateObjCommand(tclip, "show-c", showc, NULL, NULL);
	return TCL_OK;
}

int main(int argc, char * argv[]) {
	cout << "mcPolymer written by Marco Drache, Georg Drache,  (c) 2008 - 2012"
			<< endl << endl;
	cout
			<< "This program is free software: you can redistribute it and/or modify"
			<< endl;
	cout
			<< "it under the terms of the GNU General Public License as published by"
			<< endl;
	cout << "the Free Software Foundation, either version 3 of the License, or"
			<< endl;
	cout << "(at your option) any later version." << endl;
	cout << "You should have received a copy of the GNU General Public License"
			<< endl;
	cout
			<< "along with this program.  If not, see <http://www.gnu.org/licenses/>."
			<< endl << endl;
	cout << "This program is distributed in the hope that it will be useful,"
			<< endl;
	cout << "but WITHOUT ANY WARRANTY; without even the implied warranty of"
			<< endl;
	cout << "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the"
			<< endl;
	cout << "GNU General Public License for more details." << endl;
	cout << "You should have received a copy of the GNU General Public License"
			<< endl;
	cout
			<< "along with this program.  If not, see <http://www.gnu.org/licenses/>."
			<< endl;
	Tcl_Main(argc, argv, Tcl_AppInit);
	return 0;
}
int Tcl_AppInit(Tcl_Interp *tclip) {
	if (Tcl_Init(tclip) == TCL_ERROR)
		return TCL_ERROR;
	if (TCLIF_Init(tclip) == TCL_ERROR)
		return TCL_ERROR;
	return TCL_OK;
}
