// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2012 Marco Drache, Georg Drache
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

/*
 * polymer.h
 *
 *  Created on: 24.08.2008
 * 	Author: drache
 */
 
#include <vector>
#include <iostream>
#include <fstream>
#include <string>

using namespace std;

class Polymer {
public:
	Polymer(chain *c);
	Polymer(chain *c1, chain *c2);
	Polymer(int, chain **c1);
	Polymer(chain *c1, chain *c2, chain *c3);
	Polymer(chain *c1, chain *c2, chain *c3, chain *c4);

	~Polymer() {
		unsigned int i;
		if (conserveChains == false) {
			for (i = 0; i < polymerChain.size(); i++) {
				delete polymerChain[i];
			}
		}
	}
	inline void incrLength() {
		polymerChain[0]->incrLength();
	}
	inline int getLength() {
		int l = 0;
		vector<chain*>::iterator I;
		chain *cp;
		for (I = polymerChain.begin(); I != polymerChain.end(); ++I) {
			cp = *I;
			l += cp->getLength();
		}
		return l;
	}
	inline int getLength1() {
		return polymerChain[0]->getLength();
	}
	inline int getLength(int ind) {
		return polymerChain[ind]->getLength();
	}
	inline int getChainNumber() {
		return polymerChain.size();
	}
	inline chain* getChain(int ind) {
		return polymerChain[ind];
	}
	void conserveChainsOn() {
		conserveChains = true;
	}
	void conserveChainsOff() {
		conserveChains = false;
	}
	inline void addChain(chain *c1) {
		polymerChain.push_back(c1);
	}
	inline chain* deleteChain(double arand) {
		int anychain, nchains;
		chain *c1, *cend;
		nchains = polymerChain.size();
		if (nchains == 1) {
			c1 = polymerChain[0];
			polymerChain.pop_back();
			return c1;
		}
		anychain = static_cast<int>(arand * nchains);
		c1 = polymerChain[anychain];
		cend = polymerChain[nchains - 1];
		polymerChain[anychain] = cend;
		polymerChain.pop_back();
		return c1;
	}
	inline Polymer* delete2Chains() {
		Polymer *p2;
		p2 = new Polymer(polymerChain[2], polymerChain[3]);
		polymerChain.pop_back();
		polymerChain.pop_back();
		return p2;
	}
	inline Polymer* fracPolymer(double randr) {
		Polymer *p2;
		chain *c1;
		c1 = deleteChain(randr);
		p2 = new Polymer(c1);
		return p2;
	}

private:
	vector<chain*> polymerChain;
	bool conserveChains;
};

class Copolymer {
public:
	Copolymer(int cm1, int cm2) {
		countM1 = cm1;
		countM2 = cm2;
	}
	~Copolymer() {
	}
	inline void addM1() {
		++countM1;
	}
	inline void addM2() {
		++countM2;
	}
	inline int getLength() {
		return countM1 + countM2;
	}
	inline int getM1() {
		return countM1;
	}
	inline int getM2() {
		return countM2;
	}
private:
	int countM1;
	int countM2;
};

class polymerSpecies {
public:
	polymerSpecies() {
	}
	polymerSpecies(bool deadPolymer) {
		int i;
		// max. chain length = 1e6, change to extent
		for (i = 0; i < 1000000; i++) {
			chainHistogram[i] = 0;
		}
		NumberOfChains = 0.0;
	}
	polymerSpecies(double init) {
		int i;
		// max. chain length = 1e6, change to extent
		for (i = 0; i < 1000000; i++) {
			chainHistogram[i] = 0;
		}
		NumberOfChains = init;
	}
	~polymerSpecies();
	int getPolymerNumber() {
		return p.size();
	}
	inline int getHistogramPosition() {
		double randr;
		int i;
		long int index, ccl;
		do {
			randr = mtrandr();
			index = static_cast<int>(1.0 + randr * NumberOfChains);
			if (index <= NumberOfChains) {
				break;
			}
		} while (true);
		ccl = 0;
		// max. chain length = 1e6, change to extent
		for (i = 1; i < 999999; i++) {
			ccl = ccl + chainHistogram[i];
			if (ccl >= index) {
				break;
			}
		}
		return i;
	}

	inline void newP1() {
		chain * cp;
		Polymer *p1;
		cp = new chain();
		p1 = new Polymer(cp);
		p.push_back(p1);
	}

	inline void newP1_HP() {
		NumberOfChains = NumberOfChains + 1.0;
		++chainHistogram[1];
	}

	inline void incrP() {
		Polymer *p1;
		p1 = getPolymer();
		p1->incrLength();
	}

	inline void incrP_HP() {
		int hindex;
		hindex = getHistogramPosition();
		--chainHistogram[hindex];
		++chainHistogram[hindex + 1];
	}

	int getSize() {
		return p.size();
	}
	inline Polymer* deleteP() {
		Polymer *pn, *pend;
		double randr;
		long int index;
		if (p.size() == 1) {
			pn = p[0];
			p.pop_back();
			return pn;
		} else {

			do {
				randr = mtrandr();
				index = static_cast<int>(randr * p.size());
				if (index < p.size()) {
					if (index >= 0) {
						break;
					}
				}
			} while (true);

			pn = p[index];
			pend = p[p.size() - 1];
			p[index] = pend;
			p.pop_back();
			return pn;
		}
	}
	inline int deleteP_HP() {
		int hindex;
		hindex = getHistogramPosition();
		--chainHistogram[hindex];
		NumberOfChains = NumberOfChains - 1.0;
		return hindex;
	}
	inline void newPplusX(Polymer *p1, Polymer *p2) {
		int i, cn;
		// max. 10 chains are part of one macromolecule, change to extent
		chain *c12[10];
		Polymer *p11;
		c12[0] = p1->getChain(0);
		p1->conserveChainsOn();
		delete p1;
		cn = p2->getChainNumber();
		for (i = 0; i < cn; i++) {
			c12[i + 1] = p2->getChain(i);
		}
		p2->conserveChainsOn();
		delete p2;
		p11 = new Polymer(cn + 1, c12);
		p.push_back(p11);
	}
	inline void newP(Polymer *p1) {
		chain *c1;
		Polymer *p11;
		c1 = p1->getChain(0);
		p1->conserveChainsOn();
		delete p1;
		p11 = new Polymer(c1);
		p.push_back(p11);
	}
	inline void newP_HP(int hindex) {
		NumberOfChains = NumberOfChains + 1.0;
		if (hindex < 1000000) {
			chainHistogram[hindex] += 1;
		}
	}
	inline void newPfromPolymer(Polymer *p1) {
		p.push_back(p1);
	}
	inline void newDeadP(Polymer *p1, Polymer *p2) {
		unsigned int clength;
		clength = p1->getLength1() + p2->getLength1();
		p1->conserveChainsOff();
		delete p1;
		p2->conserveChainsOff();
		delete p2;
		if (clength < 1000000) {
			chainHistogram[clength] += 1;
		}
	}
	inline void newDeadP_HP(int hindex1, int hindex2) {
		unsigned int clength;
		clength = hindex1 + hindex2;
		if (clength < 1000000) {
			chainHistogram[clength] += 1;
		}
	}
	inline void newDeadP(Polymer *p1) {
		unsigned int clength;
		clength = p1->getLength1();
		p1->conserveChainsOff();
		delete p1;
		if (clength < 1000000) {
			chainHistogram[clength] += 1;
		}
	}
	inline void newDeadP_HP(int hindex) {
		if (hindex < 1000000) {
			chainHistogram[hindex] += 1;
		}
	}
	void writeDistribution(int ind1, int ind2) {
		char buffer[100];
		int slength;
		vector<Polymer*>::iterator I;
		vector<unsigned int> nChains;
		vector<unsigned int> chainLength;
		vector<unsigned int>::iterator II;
		vector<unsigned int>::iterator IIV;
		bool findLength;
		unsigned int cLength;
		unsigned int i;
		Polymer *pn;
		double snipi, snipi2, Pn, Pw;

		for (I = p.begin(); I != p.end(); ++I) {
			pn = *I;
			cLength = pn->getLength();
			findLength = false;
			II = chainLength.begin();
			IIV = nChains.begin();
			if (chainLength.size() == 0) {
				chainLength.resize(1);
				nChains.resize(1);
				chainLength[0] = cLength;
				nChains[0] = 1;
				continue;
			}
			for (i = 0; i < chainLength.size(); i++) {
				if (chainLength[i] == cLength) {
					findLength = true;
					++nChains[i];
					break;
				}
				if (chainLength[i] > cLength) {

					chainLength.insert(II, cLength);
					nChains.insert(IIV, 1);
					findLength = true;
					break;
				}
				++II;
				++IIV;
			}
			if (!findLength) {
				chainLength.push_back(cLength);
				nChains.push_back(1);
			}
		}
		snipi = 0.0;
		snipi2 = 0.0;
		for (i = 0; i < chainLength.size(); i++) {
			snipi = snipi + chainLength[i] * nChains[i];
			snipi2 = snipi2
					+ 1.0 * chainLength[i] * chainLength[i] * nChains[i];
		}

		slength = sprintf(buffer, "polymer.%u.%u", ind1, ind2);
		ofstream resFile(buffer);
		resFile << "nChains:\t" << p.size() << endl;
		Pn = snipi / p.size();
		Pw = snipi2 / snipi;
		resFile << "Pn:\t" << Pn << "\tPw:\t" << Pw << "\tPd:\t" << Pw / Pn
				<< "\tMacro" << endl;

		resFile << "chainLength\tnChains" << endl;
		for (i = 0; i < chainLength.size(); i++) {
			resFile << chainLength[i] << "\t" << nChains[i] << endl;
		}

		resFile.close();
	}
	void writeDistributionBinary(int ind1, int ind2) {
		char buffer[100];
		int slength;
		int i;
		int j, nc, cl;
		ofstream binf;

		slength = sprintf(buffer, "polymerBinary.%u.%u", ind1, ind2);
		binf.open(buffer, ios::binary | ios::out);
		if (!binf) {
			cerr << "error open file" << endl;
		}
		i = p.size();
		binf.write((char*) &i, sizeof(i));

		for (i = 0; i < p.size(); i++) {
			nc = p[i]->getChainNumber();
			binf.write((char*) &nc, sizeof(nc));
			for (j = 0; j < nc; j++) {
				cl = p[i]->getLength(j);
				binf.write((char*) &cl, sizeof(cl));
			}
		}
		binf.close();
	}
	void Defragmentation() {
		char buffer[100];
		int slength;
		int i;
		int j, nc, cl;
		ofstream binf;
		vector<Polymer*> pnew;
		chain * cp;

		pnew.resize(p.size());

		for (i = 0; i < p.size(); i++) {
			nc = p[i]->getChainNumber();
			for (j = 0; j < nc; j++) {
				cl = p[i]->getLength(j);
				cp = new chain();
				cp->setLength(cl);
				if (j == 0) {
					pnew[i] = new Polymer(cp);
				} else {
					pnew[i]->addChain(cp);
				}
			}
			p[i]->conserveChainsOff();
			delete p[i];
		}
		p.resize(0);
		p = pnew;
	}
	void writeLabeledDistributionBinary(int count, int ind1, int ind2) {
		char buffer[100];
		int slength;
		int i;
		int j, nc, cl, sizel;
		Polymer *pn, *pend;
		ofstream binf;

		slength = sprintf(buffer, "polymerBinary.%u.%u.%u", count, ind1, ind2);
		binf.open(buffer, ios::binary | ios::out);
		if (!binf) {
			cerr << "error open file" << endl;
		}
		sizel = 0;
		for (i = 0; i < p.size(); i++) {
			if (labeled[i] == true) {
				++sizel;
			}
		}
		binf.write((char*) &sizel, sizeof(sizel));
		for (i = 0; i < p.size(); i++) {
			if (labeled[i] == true) {
				nc = p[i]->getChainNumber();
				binf.write((char*) &nc, sizeof(nc));
				for (j = 0; j < nc; j++) {
					cl = p[i]->getLength(j);

					binf.write((char*) &cl, sizeof(cl));
				}

			}
		}
		binf.close();
		for (i = p.size() - 1; i >= 0; i--) {
			if (labeled[i] == true) {
				pn = p[i];
				pend = p[p.size() - 1];
				p[i] = pend;
				p.pop_back();
				pn->conserveChainsOff();
				delete pn;
			}

		}
		delete labeled;

	}
	void readDistributionBinary(int vz, int ind1, int ind2) {
		char buffer[100];
		int slength;
		int i;
		int j, nc, cl, np, ofs;
		ifstream binf;
		chain * cp;

		ofs = p.size();

		if (vz == 0) {
			slength = sprintf(buffer, "polymerBinary.%u.%u", ind1, ind2);
		} else {
			slength = sprintf(buffer, "%u/polymerBinary.%u.%u", vz, ind1, ind2);
		}

		binf.open(buffer, ios::binary | ios::in);
		if (!binf) {
			cerr << "error open file" << endl;
		}
		binf.read((char*) &np, sizeof(np));
		p.resize(p.size() + np);
		for (i = ofs; i < np + ofs; i++) {
			binf.read((char*) &nc, sizeof(nc));
			if (nc == 0) {
				cout << i << " : Mist" << endl;
				return;
			}
			for (j = 0; j < nc; j++) {
				binf.read((char*) &cl, sizeof(cl));
				cp = new chain();
				cp->setLength(cl);
				if (j == 0) {
					p[i] = new Polymer(cp);
				} else {
					p[i]->addChain(cp);
				}
			}
		}
		binf.close();
	}
	void writeDistributionDead(int ind1, int ind2) {
		char buffer[100];
		int slength;
		unsigned int cLength;
		unsigned int i;
		double sni, snipi, snipi2, Pn, Pw;

		sni = 0.0;
		snipi = 0.0;
		snipi2 = 0.0;
		Pn = 0.0;
		Pw = 0.0;
		for (i = 0; i < 1000000; i++) {
			sni = sni + chainHistogram[i];
			snipi = snipi + i * chainHistogram[i];
			snipi2 = snipi2 + 1.0 * i * i * chainHistogram[i];
		}

		slength = sprintf(buffer, "polymer.%u.%u", ind1, ind2);
		ofstream resFile(buffer);
		resFile << "nChains:\t" << sni << endl;
		if (sni > 0) {
			Pn = snipi / sni;
			Pw = snipi2 / snipi;
			resFile << "Pn:\t" << Pn << "\tPw:\t" << Pw << "\tPd:\t" << Pw / Pn
					<< "\tMacroD" << endl;
		} else {
			resFile << "Pn:\t" << Pn << "\tPw:\t" << Pw << "\tPd:\t" << 0.0
					<< "\tMacroD" << endl;
		}

		resFile << "chainLength\tnChains" << endl;
		for (i = 0; i < 1000000; i++) {
			if (chainHistogram[i] > 0) {
				resFile << i << "\t" << chainHistogram[i] << endl;
			}
		}

		resFile.close();
	}
	void writeDistributionBinaryDead(int ind1, int ind2) {
		char buffer[100];
		int slength;
		int i;
		int nz;
		ofstream binf;
		slength = sprintf(buffer, "polymerBinaryDead.%u.%u", ind1, ind2);
		binf.open(buffer, ios::binary | ios::out);
		if (!binf) {
			cerr << "error open file" << endl;
		}
		nz = 0;
		for (i = 0; i < 1000000; i++) {
			if (chainHistogram[i] > 0) {
				nz = nz + 1;
			}
		}
		binf.write((char*) &nz, sizeof(nz));
		for (i = 0; i < 1000000; i++) {
			if (chainHistogram[i] > 0) {
				binf.write((char*) &i, sizeof(i));
				binf.write((char*) &chainHistogram[i],
						sizeof(chainHistogram[i]));
			}
		}
		binf.close();
	}
	void readDistributionBinaryDead(int ind1, int ind2) {
		char buffer[100];
		int slength;
		int i;
		int nz, zi, zl;
		ifstream binf;

		slength = sprintf(buffer, "polymerBinaryDead.%u.%u", ind1, ind2);
		binf.open(buffer, ios::binary | ios::in);
		if (!binf) {
			cerr << "error open file" << endl;
		}
		binf.read((char*) &nz, sizeof(nz));
		cout << nz << "  Zeilen" << endl;

		for (i = 0; i < nz; i++) {
			binf.read((char*) &zi, sizeof(zi));
			binf.read((char*) &zl, sizeof(zl));
			chainHistogram[zi] = chainHistogram[zi] + zl;
			cout << zi << " : " << zl << endl;
		}
		binf.close();
	}
	void markAll() {
		int i;
		labeled = new bool[p.size()];
		for (i = 0; i < p.size(); i++) {
			labeled[i] = true;
		}
	}
	void mark(int count) {
		int i, j;
		double randr;
		int index;
		labeled = new bool[p.size()];
		for (i = 0; i < p.size(); i++) {
			labeled[i] = false;
		}
		for (i = 0; i < count; i++) {
			do {
				randr = mtrandr();
				index = static_cast<int>(randr * p.size());
				if (index < p.size()) {
					if (index >= 0) {
						break;
					}
				}
			} while (true);
			do {
				if (labeled[index] == false) {
					labeled[index] = true;
					break;
				} else {
					++index;
					if (index == p.size()) {
						index = 0;
					}
				}
			} while (true);

		}
	}

	// Copolymers
	inline void newCP(int cm1, int cm2) {
		Copolymer *cop;
		cop = new Copolymer(cm1, cm2);
		cp.push_back(cop);
	}
	inline void newCP(Copolymer *cop) {
		cp.push_back(cop);
	}
	inline Copolymer* deleteCP() {
		Copolymer *copn, *copend;
		double randr;
		long int index;
		if (cp.size() == 1) {
			copn = cp[0];
			cp.pop_back();
			return copn;
		} else {

			do {
				randr = mtrandr();
				index = static_cast<int>(randr * cp.size());
				if (index < cp.size()) {
					if (index >= 0) {
						break;
					}
				}
			} while (true);
			copn = cp[index];
			copend = cp[cp.size() - 1];
			cp[index] = copend;
			cp.pop_back();
			return copn;
		}
	}
	inline void newDeadCP(Copolymer *cop1, Copolymer *cop2) {
		int cm1, cm2;
		cm1 = cop1->getM1() + cop2->getM1();
		cm2 = cop1->getM2() + cop2->getM2();
		delete cop1;
		delete cop2;
		cop1 = new Copolymer(cm1, cm2);
		cp.push_back(cop1);
	}
	void writeDistributionCP(int ind1, int ind2) {
		char buffer[100];
		int slength;
		vector<Copolymer*>::iterator I;
		vector<unsigned int> nChains;
		vector<unsigned int> chainLength;
		vector<double> monomerM1;
		vector<double> monomerM2;
		vector<unsigned int>::iterator II;
		vector<unsigned int>::iterator IIV;
		vector<double>::iterator II1;
		vector<double>::iterator II2;
		bool findLength;
		unsigned int cLength;
		unsigned int i, j;
		Copolymer *pn;
		double snipi, snipi2, Pn, Pw;
		vector < vector<int> > contour;
		int maxM1, maxM2;
		maxM1 = 0;
		maxM2 = 0;

		for (I = cp.begin(); I != cp.end(); ++I) {
			pn = *I;
			cLength = pn->getLength();
			if (pn->getM1() > maxM1) {
				maxM1 = pn->getM1();
			}
			if (pn->getM2() > maxM2) {
				maxM2 = pn->getM2();
			}
			findLength = false;
			II = chainLength.begin();
			IIV = nChains.begin();
			II1 = monomerM1.begin();
			II2 = monomerM2.begin();
			if (chainLength.size() == 0) {
				chainLength.resize(1);
				nChains.resize(1);
				monomerM1.resize(1);
				monomerM2.resize(1);
				chainLength[0] = cLength;
				nChains[0] = 1;
				monomerM1[0] = pn->getM1();
				monomerM2[0] = pn->getM2();
				continue;
			}
			for (i = 0; i < chainLength.size(); i++) {
				if (chainLength[i] == cLength) {
					findLength = true;
					++nChains[i];
					monomerM1[i] = monomerM1[i] + pn->getM1();
					monomerM2[i] = monomerM2[i] + pn->getM2();
					break;
				}
				if (chainLength[i] > cLength) {

					chainLength.insert(II, cLength);
					nChains.insert(IIV, 1);
					monomerM1.insert(II1, pn->getM1());
					monomerM2.insert(II2, pn->getM2());
					findLength = true;
					break;
				}
				++II;
				++IIV;
				++II1;
				++II2;
			}
			if (!findLength) {
				chainLength.push_back(cLength);
				nChains.push_back(1);
				monomerM1.push_back(pn->getM1());
				monomerM2.push_back(pn->getM2());
			}
		}
		snipi = 0.0;
		snipi2 = 0.0;
		for (i = 0; i < chainLength.size(); i++) {
			snipi = snipi + chainLength[i] * nChains[i];
			snipi2 = snipi2
					+ 1.0 * chainLength[i] * chainLength[i] * nChains[i];
		}

		slength = sprintf(buffer, "polymer.%u.%u", ind1, ind2);
		ofstream resFile(buffer);
		resFile << "nChains:\t" << cp.size() << endl;
		Pn = snipi / cp.size();
		Pw = snipi2 / snipi;
		resFile << "Pn:\t" << Pn << "\tPw:\t" << Pw << "\tPd:\t" << Pw / Pn
				<< "\tMacro" << endl;

		resFile << "chainLength\tnChains\tnM1\tnM2" << endl;
		for (i = 0; i < chainLength.size(); i++) {
			resFile << chainLength[i] << "\t" << nChains[i] << "\t"
					<< monomerM1[i] << "\t" << monomerM2[i] << endl;
		}
		resFile.close();
		contour.resize(maxM1 + 1);
		for (i = 0; i <= maxM1; i++) {
			contour[i].resize(maxM2 + 1);
			for (j = 0; j <= maxM2; j++) {
				contour[i][j] = 0;
			}
		}
		for (I = cp.begin(); I != cp.end(); ++I) {
			pn = *I;
			++contour[pn->getM1()][pn->getM2()];
		}
		slength = sprintf(buffer, "polymerContour.%u.%u", ind1, ind2);
		ofstream resFile1(buffer);
		resFile1 << "nChains:\t" << cp.size() << "\t" << maxM1 << "\tX\t"
				<< maxM2 << endl;
		resFile1 << "\t";
		for (j = 0; j <= maxM2; j++) {
			resFile1 << j << "\t";
		}
		resFile1 << endl;
		for (i = 0; i <= maxM1; i++) {
			resFile1 << i << "\t";
			for (j = 0; j <= maxM2; j++) {
				resFile1 << contour[i][j] << "\t";
			}
			resFile1 << endl;
		}
		resFile1.close();
	}
private:
	inline Polymer* getPolymer() {
		double randr;
		int index;
		do {
			randr = mtrandr();
			index = static_cast<int>(randr * p.size());
			if (index < p.size()) {
				if (index >= 0) {
					break;
				}
			}
		} while (true);
		return p[index];
	}
	inline Copolymer* getCopolymer() {
		double randr;
		int index;
		do {
			randr = mtrandr();
			index = static_cast<int>(randr * cp.size());
			if (index < cp.size()) {
				if (index >= 0) {
					break;
				}
			}
		} while (true);
		return cp[index];
	}
	vector<Polymer*> p;
	vector<Copolymer*> cp;
	bool *labeled;
	MTRand mtrandr;
	double NumberOfChains;
	// max. chain length = 1e6, change to extent
	unsigned int chainHistogram[1000000];
};
